@@ -25,15 +25,14 @@ from dataclasses import dataclass
 from enum import Enum, unique
 from typing import Any, Callable
 
-import jwt
-
 from livekit import api, rtc
+from livekit.api.access_token import Claims
 from livekit.protocol import agent, models
 
 from .cli import cli
 from .ipc.inference_executor import InferenceExecutor
 from .log import logger
-from .types import NOT_GIVEN, NotGivenOr
+from .types import NotGivenOr
 from .utils import http_context, is_given, wait_for_participant
 
 _JobContextVar = contextvars.ContextVar["JobContext"]("agents_job_context")
@@ -432,12 +431,8 @@ class JobContext:
                 lambda _, coro=coro: self._participant_tasks.pop((p.identity, coro))  # type: ignore
             )
 
-    def decode_token(self, api_secret: NotGivenOr[str] = NOT_GIVEN) -> dict[str, Any]:
-        options = {}
-        if not is_given(api_secret):
-            options["verify_signature"] = False
-            api_secret = ""
-        return jwt.decode(self._info.token, api_secret, options=options, algorithms=["HS256"])  # type: ignore
+    def token_claims(self) -> Claims:
+        return api.TokenVerifier().verify(self._info.token, verify_signature=False)
 
 
 def _apply_auto_subscribe_opts(room: rtc.Room, auto_subscribe: AutoSubscribe) -> None:
