@@ -1,4 +1,4 @@
-from . import http_server, metrics, trace_types
+from . import http_server, metrics, trace_types, utils
 from .traces import set_tracer_provider, tracer
 
 __all__ = [
@@ -7,6 +7,7 @@ __all__ = [
     "trace_types",
     "http_server",
     "set_tracer_provider",
+    "utils",
 ]
 
 # Cleanup docs of unexported modules
