@@ -1,8 +1,10 @@
+from . import io
 from .agent import Agent, AgentTask, ModelSettings
 from .agent_session import AgentSession, VoiceActivityVideoSampler
 from .chat_cli import ChatCLI
 from .events import (
     AgentEvent,
+    AgentFalseInterruptionEvent,
     AgentStateChangedEvent,
     CloseEvent,
     CloseReason,
@@ -15,7 +17,13 @@ from .events import (
     UserInputTranscribedEvent,
     UserStateChangedEvent,
 )
+from .room_io import (
+    _ParticipantAudioOutput,
+    _ParticipantStreamTranscriptionOutput,
+    _ParticipantTranscriptionOutput,
+)
 from .speech_handle import SpeechHandle
+from .transcription import TranscriptSynchronizer
 
 __all__ = [
     "ChatCLI",
@@ -37,6 +45,13 @@ __all__ = [
     "UserStateChangedEvent",
     "AgentStateChangedEvent",
     "FunctionToolsExecutedEvent",
+    "AgentFalseInterruptionEvent",
+    "TranscriptSynchronizer",
+    "io",
+    "room_io",
+    "_ParticipantAudioOutput",
+    "_ParticipantTranscriptionOutput",
+    "_ParticipantStreamTranscriptionOutput",
 ]
 
 # Cleanup docs of unexported modules
