@@ -39,12 +39,14 @@ from .agent import Agent
 from .agent_activity import AgentActivity
 from .audio_recognition import _TurnDetector
 from .events import (
+    AgentFalseInterruptionEvent,
     AgentState,
     AgentStateChangedEvent,
     CloseEvent,
     CloseReason,
     ConversationItemAddedEvent,
     EventTypes,
+    UserInputTranscribedEvent,
     UserState,
     UserStateChangedEvent,
 )
@@ -74,6 +76,7 @@ class VoiceOptions:
     max_endpointing_delay: float
     max_tool_steps: int
     user_away_timeout: float | None
+    agent_false_interruption_timeout: float | None
     min_consecutive_speech_delay: float
     use_tts_aligned_transcript: bool
     preemptive_generation: bool
@@ -148,11 +151,12 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         discard_audio_if_uninterruptible: bool = True,
         min_interruption_duration: float = 0.5,
         min_interruption_words: int = 0,
-        min_endpointing_delay: float = 0.5,
+        min_endpointing_delay: float = 0.4,
         max_endpointing_delay: float = 6.0,
         max_tool_steps: int = 3,
         video_sampler: NotGivenOr[_VideoSampler | None] = NOT_GIVEN,
         user_away_timeout: float | None = 15.0,
+        agent_false_interruption_timeout: float | None = 4.0,
         min_consecutive_speech_delay: float = 0.0,
         use_tts_aligned_transcript: bool = False,
         preemptive_generation: bool = False,
@@ -202,7 +206,7 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             min_endpointing_delay (float): Minimum time-in-seconds the agent
                 must wait after a potential end-of-utterance signal (from VAD
                 or an EOU model) before it declares the user’s turn complete.
-                Default ``0.5`` s.
+                Default ``0.4`` s.
             max_endpointing_delay (float): Maximum time-in-seconds the agent
                 will wait before terminating the turn. Default ``6.0`` s.
             max_tool_steps (int): Maximum consecutive tool calls per LLM turn.
@@ -214,6 +218,10 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             user_away_timeout (float, optional): If set, set the user state as
                 "away" after this amount of time after user and agent are silent.
                 Default ``15.0`` s, set to ``None`` to disable.
+            agent_false_interruption_timeout (float, optional): If set, emit an
+                `agent_false_interruption` event after this amount of time if
+                the user is silent and no user transcript is detected after
+                the interruption. Set to ``None`` to disable. Default ``4.0`` s.
             min_consecutive_speech_delay (float, optional): The minimum delay between
                 consecutive speech. Default ``0.0`` s.
             use_tts_aligned_transcript (bool, optional): Whether to use TTS-aligned
@@ -253,6 +261,7 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             max_endpointing_delay=max_endpointing_delay,
             max_tool_steps=max_tool_steps,
             user_away_timeout=user_away_timeout,
+            agent_false_interruption_timeout=agent_false_interruption_timeout,
             min_consecutive_speech_delay=min_consecutive_speech_delay,
             preemptive_generation=preemptive_generation,
             use_tts_aligned_transcript=use_tts_aligned_transcript,
@@ -295,6 +304,10 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         self._agent_state: AgentState = "initializing"
         self._user_away_timer: asyncio.TimerHandle | None = None
 
+        # used to emit the agent false interruption event
+        self._false_interruption_timer: asyncio.TimerHandle | None = None
+        self._false_interrupted_event: AgentFalseInterruptionEvent | None = None
+
         self._userdata: Userdata_T | None = userdata if is_given(userdata) else None
         self._closing_task: asyncio.Task[None] | None = None
         self._job_context_cb_registered: bool = False
@@ -400,7 +413,7 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             self._root_span_context = otel_context.get_current()
             self._session_span = current_span = trace.get_current_span()
             current_span = trace.get_current_span()
-            current_span.set_attribute(trace_types.ATTR_AGENT_NAME, agent.label)
+            current_span.set_attribute(trace_types.ATTR_AGENT_LABEL, agent.label)
             current_span.set_attribute(
                 trace_types.ATTR_SESSION_OPTIONS, json.dumps(asdict(self._opts))
             )
@@ -512,6 +525,42 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
 
                 self._room_io.subscribed_fut.add_done_callback(on_room_io_subscribed)
 
+            # log used IO
+            def _collect_source(
+                inp: io.AudioInput | io.VideoInput | None,
+            ) -> list[io.AudioInput | io.VideoInput]:
+                return [] if inp is None else [inp] + _collect_source(inp.source)
+
+            def _collect_chain(
+                out: io.TextOutput | io.VideoOutput | io.AudioOutput | None,
+            ) -> list[io.VideoOutput | io.AudioOutput | io.TextOutput]:
+                return [] if out is None else [out] + _collect_chain(out.next_in_chain)
+
+            audio_input = _collect_source(self.input.audio)[::-1]
+            video_input = _collect_source(self.input.video)[::-1]
+
+            audio_output = _collect_chain(self.output.audio)
+            video_output = _collect_chain(self.output.video)
+            transcript_output = _collect_chain(self.output.transcription)
+
+            logger.debug(
+                "using audio io: %s -> `AgentSession` -> %s",
+                " -> ".join([f"`{out.label}`" for out in audio_input]) or "(none)",
+                " -> ".join([f"`{out.label}`" for out in audio_output]) or "(none)",
+            )
+
+            logger.debug(
+                "using transcript io: `AgentSession` -> %s",
+                " -> ".join([f"`{out.label}`" for out in transcript_output]) or "(none)",
+            )
+
+            if video_input or video_output:
+                logger.debug(
+                    "using video io: %s > `AgentSession` > %s",
+                    " -> ".join([f"`{out.label}`" for out in video_input]) or "(none)",
+                    " -> ".join([f"`{out.label}`" for out in video_output]) or "(none)",
+                )
+
     async def drain(self) -> None:
         if self._activity is None:
             raise RuntimeError("AgentSession isn't running")
@@ -540,6 +589,10 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         drain: bool = False,
         error: llm.LLMError | stt.STTError | tts.TTSError | llm.RealtimeModelError | None = None,
     ) -> None:
+        if self._root_span_context:
+            # make `activity.drain` and `on_exit` under the root span
+            otel_context.attach(self._root_span_context)
+
         async with self._lock:
             if not self._started:
                 return
@@ -591,6 +644,7 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             self.emit("close", CloseEvent(error=error, reason=reason))
 
             self._cancel_user_away_timer()
+            self._cancel_agent_false_interruption()
             self._user_state = "listening"
             self._agent_state = "initializing"
             self._llm_error_counts = 0
@@ -900,6 +954,9 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         else:
             self._cancel_user_away_timer()
 
+        if state != "listening":
+            self._cancel_agent_false_interruption()
+
         old_state = self._agent_state
         self._agent_state = state
         self.emit(
@@ -927,6 +984,14 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         else:
             self._cancel_user_away_timer()
 
+        # pause the false interruption timer if user is speaking and recreate it after user stops
+        if state == "speaking" and self._false_interruption_timer:
+            ev = self._false_interrupted_event
+            self._cancel_agent_false_interruption()
+            self._false_interrupted_event = ev
+        elif state == "listening" and self._false_interrupted_event:
+            self._schedule_agent_false_interruption(self._false_interrupted_event)
+
         old_state = self._user_state
         self._user_state = state
         self.emit(
@@ -934,10 +999,39 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             UserStateChangedEvent(old_state=old_state, new_state=state),
         )
 
+    def _user_input_transcribed(self, ev: UserInputTranscribedEvent) -> None:
+        self.emit("user_input_transcribed", ev)
+        if ev.is_final:
+            # fully cancel the false interruption event if user transcript arrives
+            self._cancel_agent_false_interruption()
+
     def _conversation_item_added(self, message: llm.ChatMessage) -> None:
         self._chat_ctx.insert(message)
         self.emit("conversation_item_added", ConversationItemAddedEvent(item=message))
 
+    def _schedule_agent_false_interruption(self, ev: AgentFalseInterruptionEvent) -> None:
+        if self._opts.agent_false_interruption_timeout is None:
+            return
+
+        def _emit_event() -> None:
+            if self._agent_state != "listening" or self._user_state != "listening":
+                return
+
+            self.emit("agent_false_interruption", ev)
+            self._false_interruption_timer = None
+
+        self._cancel_agent_false_interruption()
+        self._false_interruption_timer = self._loop.call_later(
+            self._opts.agent_false_interruption_timeout, _emit_event
+        )
+        self._false_interrupted_event = ev
+
+    def _cancel_agent_false_interruption(self) -> None:
+        if self._false_interruption_timer is not None:
+            self._false_interruption_timer.cancel()
+            self._false_interruption_timer = None
+        self._false_interrupted_event = None
+
     # move them to the end to avoid shadowing the same named modules for mypy
     @property
     def stt(self) -> stt.STT | None:
