@@ -3,9 +3,9 @@ from __future__ import annotations
 import asyncio
 import json
 import math
-from collections.abc import AsyncGenerator, AsyncIterator
+from collections.abc import AsyncIterator
 from dataclasses import asdict
-from typing import Any
+from typing import Any, Callable, Union
 
 from livekit import rtc
 
@@ -25,6 +25,9 @@ class DataStreamAudioOutput(AudioOutput):
     AudioOutput implementation that streams audio to a remote avatar worker using LiveKit DataStream.
     """  # noqa: E501
 
+    _playback_finished_handlers: dict[str, Callable[[rtc.RpcInvocationData], str]] = {}
+    _playback_finished_rpc_registered: bool = False
+
     def __init__(
         self,
         room: rtc.Room,
@@ -33,7 +36,7 @@ class DataStreamAudioOutput(AudioOutput):
         sample_rate: int | None = None,
         wait_remote_track: rtc.TrackKind.ValueType | None = None,
     ):
-        super().__init__(next_in_chain=None, sample_rate=sample_rate)
+        super().__init__(label="DataStreamIO", next_in_chain=None, sample_rate=sample_rate)
         self._room = room
         self._destination_identity = destination_identity
         self._wait_remote_track = wait_remote_track
@@ -58,8 +61,10 @@ class DataStreamAudioOutput(AudioOutput):
 
             await self._room_connected_fut
 
-            self._room.local_participant.register_rpc_method(
-                RPC_PLAYBACK_FINISHED, self._handle_playback_finished
+            self._register_playback_finished_rpc(
+                self._room,
+                caller_identity=self._destination_identity,
+                handler=self._handle_playback_finished,
             )
             logger.debug(
                 "waiting for the remote participant",
@@ -148,6 +153,11 @@ class DataStreamAudioOutput(AudioOutput):
             )
             return "reject"
 
+        logger.info(
+            "playback finished event received",
+            extra={"caller_identity": data.caller_identity},
+        )
+
         event = PlaybackFinishedEvent(**json.loads(data.payload))
         self.on_playback_finished(
             playback_position=event.playback_position,
@@ -159,6 +169,34 @@ class DataStreamAudioOutput(AudioOutput):
         if self._room.isconnected() and not self._room_connected_fut.done():
             self._room_connected_fut.set_result(None)
 
+    @classmethod
+    def _register_playback_finished_rpc(
+        cls,
+        room: rtc.Room,
+        *,
+        caller_identity: str,
+        handler: Callable[[rtc.RpcInvocationData], str],
+    ) -> None:
+        cls._playback_finished_handlers[caller_identity] = handler
+
+        if cls._playback_finished_rpc_registered:
+            return
+
+        def _handler(data: rtc.RpcInvocationData) -> str:
+            if data.caller_identity not in cls._playback_finished_handlers:
+                logger.warning(
+                    "playback finished event received from unexpected participant",
+                    extra={
+                        "caller_identity": data.caller_identity,
+                        "expected_identities": list(cls._playback_finished_handlers.keys()),
+                    },
+                )
+                return "reject"
+            return cls._playback_finished_handlers[data.caller_identity](data)
+
+        room.local_participant.register_rpc_method(RPC_PLAYBACK_FINISHED, _handler)
+        cls._playback_finished_rpc_registered = True
+
 
 class DataStreamAudioReceiver(AudioReceiver):
     """
@@ -167,12 +205,16 @@ class DataStreamAudioReceiver(AudioReceiver):
     subscribe to the first agent participant in the room.
     """
 
+    _clear_buffer_rpc_registered: bool = False
+    _clear_buffer_handlers: dict[str, Callable[[rtc.RpcInvocationData], str]] = {}
+
     def __init__(
         self,
         room: rtc.Room,
         *,
         sender_identity: str | None = None,
         frame_size_ms: NotGivenOr[int] = NOT_GIVEN,
+        rpc_max_retries: int = 3,
     ):
         super().__init__()
         self._room = room
@@ -182,10 +224,18 @@ class DataStreamAudioReceiver(AudioReceiver):
 
         self._stream_readers: list[rtc.ByteStreamReader] = []
         self._stream_reader_changed: asyncio.Event = asyncio.Event()
+        self._data_ch = utils.aio.Chan[Union[rtc.AudioFrame, AudioSegmentEnd]]()
 
         self._current_reader: rtc.ByteStreamReader | None = None
         self._current_reader_cleared: bool = False
 
+        self._playback_finished_ch = utils.aio.Chan[PlaybackFinishedEvent]()
+        self._rpc_max_retries = rpc_max_retries
+
+        self._main_atask: asyncio.Task | None = None
+        self._exception: Exception | None = None
+        self._closing: bool = False
+
     async def start(self) -> None:
         # wait for the target participant or first agent participant to join
         self._remote_participant = await utils.wait_for_participant(
@@ -193,6 +243,7 @@ class DataStreamAudioReceiver(AudioReceiver):
             identity=self._sender_identity,
             kind=rtc.ParticipantKind.PARTICIPANT_KIND_AGENT if not self._sender_identity else None,
         )
+        self._main_atask = asyncio.create_task(self._main_task())
 
         def _handle_clear_buffer(data: rtc.RpcInvocationData) -> str:
             assert self._remote_participant is not None
@@ -211,8 +262,6 @@ class DataStreamAudioReceiver(AudioReceiver):
             self.emit("clear_buffer")
             return "ok"
 
-        self._room.local_participant.register_rpc_method(RPC_CLEAR_BUFFER, _handle_clear_buffer)
-
         def _handle_stream_received(
             reader: rtc.ByteStreamReader, remote_participant_id: str
         ) -> None:
@@ -225,33 +274,64 @@ class DataStreamAudioReceiver(AudioReceiver):
             self._stream_readers.append(reader)
             self._stream_reader_changed.set()
 
+        self._register_clear_buffer_rpc(
+            self._room,
+            caller_identity=self._remote_participant.identity,
+            handler=_handle_clear_buffer,
+        )
         self._room.register_byte_stream_handler(AUDIO_STREAM_TOPIC, _handle_stream_received)
 
-    async def notify_playback_finished(self, playback_position: float, interrupted: bool) -> None:
-        """Notify the sender that playback has finished"""
-        assert self._remote_participant is not None
-        event = PlaybackFinishedEvent(playback_position=playback_position, interrupted=interrupted)
-        try:
-            logger.debug(
-                f"notifying playback finished: {event.playback_position:.3f}s, "
-                f"interrupted: {event.interrupted}"
-            )
-            await self._room.local_participant.perform_rpc(
-                destination_identity=self._remote_participant.identity,
-                method=RPC_PLAYBACK_FINISHED,
-                payload=json.dumps(asdict(event)),
-            )
-        except Exception as e:
-            logger.exception(f"error notifying playback finished: {e}")
+    def notify_playback_finished(self, playback_position: float, interrupted: bool) -> None:
+        self._playback_finished_ch.send_nowait(
+            PlaybackFinishedEvent(playback_position=playback_position, interrupted=interrupted)
+        )
 
-    def __aiter__(self) -> AsyncIterator[rtc.AudioFrame | AudioSegmentEnd]:
-        return self._stream_impl()
+    async def _main_task(self) -> None:
+        tasks = [
+            asyncio.create_task(self._recv_task()),
+            asyncio.create_task(self._send_task()),
+        ]
+        try:
+            await asyncio.gather(*tasks)
+        except Exception as error:
+            self._exception = error
+        finally:
+            self._playback_finished_ch.close()
+            self._data_ch.close()
+            await utils.aio.cancel_and_wait(*tasks)
 
     @utils.log_exceptions(logger=logger)
-    async def _stream_impl(
-        self,
-    ) -> AsyncGenerator[rtc.AudioFrame | AudioSegmentEnd, None]:
-        while True:
+    async def _send_task(self) -> None:
+        async for event in self._playback_finished_ch:
+            assert self._remote_participant is not None
+
+            retry_count = 0  # TODO: use retry logic in rust
+            while retry_count < self._rpc_max_retries:
+                logger.debug(
+                    f"notifying playback finished: {event.playback_position:.3f}s, "
+                    f"interrupted: {event.interrupted}"
+                )
+                try:
+                    await self._room.local_participant.perform_rpc(
+                        destination_identity=self._remote_participant.identity,
+                        method=RPC_PLAYBACK_FINISHED,
+                        payload=json.dumps(asdict(event)),
+                    )
+                    break
+                except rtc.RpcError as e:
+                    if retry_count == self._rpc_max_retries - 1:
+                        logger.error(
+                            f"failed to notify playback finished after {retry_count + 1} retries",
+                            exc_info=e,
+                        )
+                        raise
+                    retry_count += 1
+                    logger.warning("failed to notify the agent playback finished, retrying...")
+                    await asyncio.sleep(0.1)
+
+    @utils.log_exceptions(logger=logger)
+    async def _recv_task(self) -> None:
+        while not self._data_ch.closed:
             await self._stream_reader_changed.wait()
 
             while self._stream_readers:
@@ -271,19 +351,74 @@ class DataStreamAudioReceiver(AudioReceiver):
                     num_channels=num_channels,
                     samples_per_channel=int(math.ceil(sample_rate * self._frame_size_ms / 1000)),
                 )
-                async for data in self._current_reader:
-                    if self._current_reader_cleared:
-                        # ignore the rest data of the current reader if clear_buffer was called
-                        break
-                    for frame in bstream.push(data):
-                        yield frame
 
-                if not self._current_reader_cleared:
-                    for frame in bstream.flush():
-                        yield frame
+                try:
+                    async for data in self._current_reader:
+                        if self._current_reader_cleared:
+                            # ignore the rest data of the current reader if clear_buffer was called
+                            break
+                        for frame in bstream.push(data):
+                            self._data_ch.send_nowait(frame)
 
-                self._current_reader = None
-                self._current_reader_cleared = False
-                yield AudioSegmentEnd()
+                    if not self._current_reader_cleared:
+                        for frame in bstream.flush():
+                            self._data_ch.send_nowait(frame)
+
+                    self._current_reader = None
+                    self._current_reader_cleared = False
+                    self._data_ch.send_nowait(AudioSegmentEnd())
+
+                except utils.aio.ChanClosed:
+                    if self._closing:
+                        return
+                    raise
 
             self._stream_reader_changed.clear()
+
+    def __aiter__(self) -> AsyncIterator[rtc.AudioFrame | AudioSegmentEnd]:
+        return self
+
+    async def __anext__(self) -> rtc.AudioFrame | AudioSegmentEnd:
+        try:
+            return await self._data_ch.recv()
+        except utils.aio.ChanClosed as e:
+            if self._exception:
+                raise self._exception from e
+
+            raise StopAsyncIteration from None
+
+    async def aclose(self) -> None:
+        self._closing = True
+        self._playback_finished_ch.close()
+        self._data_ch.close()
+        self._stream_reader_changed.set()
+        if self._main_atask:
+            await utils.aio.cancel_and_wait(self._main_atask)
+
+    @classmethod
+    def _register_clear_buffer_rpc(
+        cls,
+        room: rtc.Room,
+        *,
+        caller_identity: str,
+        handler: Callable[[rtc.RpcInvocationData], str],
+    ) -> None:
+        cls._clear_buffer_handlers[caller_identity] = handler
+
+        if cls._clear_buffer_rpc_registered:
+            return
+
+        def _handler(data: rtc.RpcInvocationData) -> str:
+            if data.caller_identity not in cls._clear_buffer_handlers:
+                logger.warning(
+                    "clear buffer event received from unexpected participant",
+                    extra={
+                        "caller_identity": data.caller_identity,
+                        "expected_identities": list(cls._clear_buffer_handlers.keys()),
+                    },
+                )
+                return "reject"
+            return cls._clear_buffer_handlers[data.caller_identity](data)
+
+        room.local_participant.register_rpc_method(RPC_CLEAR_BUFFER, _handler)
+        cls._clear_buffer_rpc_registered = True
