@@ -41,6 +41,8 @@ class RunContext(Generic[Userdata_T]):
         self._speech_handle = speech_handle
         self._function_call = function_call
 
+        self._initial_step_idx = speech_handle.num_steps - 1
+
     @property
     def session(self) -> AgentSession[Userdata_T]:
         return self._session
@@ -68,12 +70,22 @@ class RunContext(Generic[Userdata_T]):
         """
         self.speech_handle.allow_interruptions = False
 
+    async def wait_for_playout(self) -> None:
+        """Waits for the speech playout corresponding to this function call step.
+
+        Unlike `SpeechHandle.wait_for_playout`, which waits for the full
+        assistant turn to complete (including all function tools),
+        this method only waits for the assistant's spoken response prior running
+        this tool to finish playing."""
+        await self.speech_handle._wait_for_generation(step_idx=self._initial_step_idx)
+
 
 EventTypes = Literal[
     "user_state_changed",
     "agent_state_changed",
     "user_input_transcribed",
     "conversation_item_added",
+    "agent_false_interruption",
     "function_tools_executed",
     "metrics_collected",
     "speech_created",
@@ -107,6 +119,17 @@ class UserInputTranscribedEvent(BaseModel):
     created_at: float = Field(default_factory=time.time)
 
 
+class AgentFalseInterruptionEvent(BaseModel):
+    type: Literal["agent_false_interruption"] = "agent_false_interruption"
+    message: ChatMessage | None
+    """The `assistant` message that got interrupted"""
+    extra_instructions: str | None = None
+    """Optional instructions originally passed to `AgentSession.generate_reply` via the `instructions` argument.
+    Populated only if the user interrupted a speech response generated using `session.generate_reply`.
+    Useful for understanding what the agent was attempting to convey before the interruption."""
+    created_at: float = Field(default_factory=time.time)
+
+
 class MetricsCollectedEvent(BaseModel):
     type: Literal["metrics_collected"] = "metrics_collected"
     metrics: AgentMetrics
@@ -182,6 +205,7 @@ AgentEvent = Annotated[
         UserInputTranscribedEvent,
         UserStateChangedEvent,
         AgentStateChangedEvent,
+        AgentFalseInterruptionEvent,
         MetricsCollectedEvent,
         ConversationItemAddedEvent,
         FunctionToolsExecutedEvent,
