@@ -1,4 +1,9 @@
 from ...types import ATTRIBUTE_PUBLISH_ON_BEHALF
+from ._output import (
+    _ParticipantAudioOutput,
+    _ParticipantStreamTranscriptionOutput,
+    _ParticipantTranscriptionOutput,
+)
 from .room_io import (
     DEFAULT_ROOM_INPUT_OPTIONS,
     DEFAULT_ROOM_OUTPUT_OPTIONS,
@@ -16,6 +21,9 @@ __all__ = [
     "RoomOutputOptions",
     "ATTRIBUTE_PUBLISH_ON_BEHALF",
     "TextInputEvent",
+    "_ParticipantTranscriptionOutput",
+    "_ParticipantAudioOutput",
+    "_ParticipantStreamTranscriptionOutput",
 ]
 
 # Cleanup docs of unexported modules
