@@ -25,10 +25,12 @@ class _ParticipantAudioOutput(io.AudioOutput):
         sample_rate: int,
         num_channels: int,
         track_publish_options: rtc.TrackPublishOptions,
+        track_name: str = "roomio_audio",
         queue_size_ms: int = 100_000,  # TODO(long): move buffer to python
     ) -> None:
-        super().__init__(next_in_chain=None, sample_rate=sample_rate)
+        super().__init__(label="RoomIO", next_in_chain=None, sample_rate=sample_rate)
         self._room = room
+        self._track_name = track_name
         self._lock = asyncio.Lock()
         self._audio_source = rtc.AudioSource(sample_rate, num_channels, queue_size_ms)
         self._publish_options = track_publish_options
@@ -45,7 +47,7 @@ class _ParticipantAudioOutput(io.AudioOutput):
 
     async def _publish_track(self) -> None:
         async with self._lock:
-            track = rtc.LocalAudioTrack.create_audio_track("roomio_audio", self._audio_source)
+            track = rtc.LocalAudioTrack.create_audio_track(self._track_name, self._audio_source)
             self._publication = await self._room.local_participant.publish_track(
                 track, self._publish_options
             )
@@ -128,7 +130,7 @@ class _ParticipantAudioOutput(io.AudioOutput):
         self._republish_task = asyncio.create_task(self._publish_track())
 
 
-class _ParticipantLegacyTranscriptionOutput(io.TextOutput):
+class _ParticipantLegacyTranscriptionOutput:
     def __init__(
         self,
         room: rtc.Room,
@@ -136,7 +138,6 @@ class _ParticipantLegacyTranscriptionOutput(io.TextOutput):
         is_delta_stream: bool = True,
         participant: rtc.Participant | str | None = None,
     ):
-        super().__init__(next_in_chain=None)
         self._room, self._is_delta_stream = room, is_delta_stream
         self._track_id: str | None = None
         self._participant_identity: str | None = None
@@ -275,18 +276,19 @@ class _ParticipantLegacyTranscriptionOutput(io.TextOutput):
         return False
 
 
-class _ParticipantTranscriptionOutput(io.TextOutput):
+class _ParticipantStreamTranscriptionOutput:
     def __init__(
         self,
         room: rtc.Room,
         *,
         is_delta_stream: bool = True,
         participant: rtc.Participant | str | None = None,
+        attributes: dict[str, str] | None = None,
     ):
-        super().__init__(next_in_chain=None)
         self._room, self._is_delta_stream = room, is_delta_stream
         self._track_id: str | None = None
         self._participant_identity: str | None = None
+        self._additional_attributes = attributes or {}
 
         self._writer: rtc.TextStreamWriter | None = None
 
@@ -334,6 +336,10 @@ class _ParticipantTranscriptionOutput(io.TextOutput):
                 attributes[ATTRIBUTE_TRANSCRIPTION_TRACK_ID] = self._track_id
         attributes[ATTRIBUTE_TRANSCRIPTION_SEGMENT_ID] = self._current_id
 
+        for key, val in self._additional_attributes.items():
+            if key not in attributes:
+                attributes[key] = val
+
         return await self._room.local_participant.stream_text(
             topic=TOPIC_TRANSCRIPTION,
             sender_identity=self._participant_identity,
@@ -369,9 +375,7 @@ class _ParticipantTranscriptionOutput(io.TextOutput):
             logger.warning("failed to publish transcription", exc_info=e)
 
     async def _flush_task(self, writer: rtc.TextStreamWriter | None) -> None:
-        attributes = {
-            ATTRIBUTE_TRANSCRIPTION_FINAL: "true",
-        }
+        attributes = {ATTRIBUTE_TRANSCRIPTION_FINAL: "true"}
         if self._track_id:
             attributes[ATTRIBUTE_TRANSCRIPTION_TRACK_ID] = self._track_id
 
@@ -420,16 +424,45 @@ class _ParticipantTranscriptionOutput(io.TextOutput):
 
 
 # Keep this utility private for now
-class _ParallelTextOutput(io.TextOutput):
+class _ParticipantTranscriptionOutput(io.TextOutput):
     def __init__(
-        self, sinks: list[io.TextOutput], *, next_in_chain: io.TextOutput | None = None
+        self,
+        *,
+        room: rtc.Room,
+        is_delta_stream: bool = True,
+        participant: rtc.Participant | str | None = None,
+        next_in_chain: io.TextOutput | None = None,
     ) -> None:
-        super().__init__(next_in_chain=next_in_chain)
-        self._sinks = sinks
+        super().__init__(label="RoomIO", next_in_chain=next_in_chain)
+
+        self.__outputs: list[
+            _ParticipantLegacyTranscriptionOutput | _ParticipantStreamTranscriptionOutput
+        ] = [
+            _ParticipantLegacyTranscriptionOutput(
+                room=room,
+                is_delta_stream=is_delta_stream,
+                participant=participant,
+            ),
+            _ParticipantStreamTranscriptionOutput(
+                room=room,
+                is_delta_stream=is_delta_stream,
+                participant=participant,
+            ),
+        ]
+
+    def set_participant(self, participant: rtc.Participant | str | None) -> None:
+        for source in self.__outputs:
+            source.set_participant(participant)
 
     async def capture_text(self, text: str) -> None:
-        await asyncio.gather(*[sink.capture_text(text) for sink in self._sinks])
+        await asyncio.gather(*[sink.capture_text(text) for sink in self.__outputs])
+
+        if self.next_in_chain:
+            await self.next_in_chain.capture_text(text)
 
     def flush(self) -> None:
-        for sink in self._sinks:
-            sink.flush()
+        for source in self.__outputs:
+            source.flush()
+
+        if self.next_in_chain:
+            self.next_in_chain.flush()
