@@ -141,6 +141,9 @@ class _WorkerEnvOption(Generic[T]):
         return opt
 
 
+_default_load_threshold = _WorkerEnvOption(dev_default=math.inf, prod_default=0.7)
+
+
 # NOTE: this object must be pickle-able
 @dataclass
 class WorkerOptions:
@@ -156,12 +159,10 @@ class WorkerOptions:
     """Called to determine the current load of the worker. Should return a value between 0 and 1."""
     job_executor_type: JobExecutorType = _default_job_executor_type
     """Which executor to use to run jobs. (currently thread or process are supported)"""
-    load_threshold: float | _WorkerEnvOption[float] = _WorkerEnvOption(
-        dev_default=math.inf, prod_default=0.75
-    )
+    load_threshold: float | _WorkerEnvOption[float] = _default_load_threshold
     """When the load exceeds this threshold, the worker will be marked as unavailable.
 
-    Defaults to 0.75 on "production" mode, and is disabled in "development" mode.
+    Defaults to 0.7 on "production" mode, and is disabled in "development" mode.
     """
 
     job_memory_warn_mb: float = 500
@@ -174,7 +175,7 @@ class WorkerOptions:
     drain_timeout: int = 1800
     """Number of seconds to wait for current jobs to finish upon receiving TERM or INT signal."""
     num_idle_processes: int | _WorkerEnvOption[int] = _WorkerEnvOption(
-        dev_default=0, prod_default=math.ceil(get_cpu_monitor().cpu_count())
+        dev_default=0, prod_default=min(math.ceil(get_cpu_monitor().cpu_count()), 4)
     )
     """Number of idle processes to keep warm."""
     shutdown_process_timeout: float = 60.0
@@ -278,6 +279,18 @@ class Worker(utils.EventEmitter[EventTypes]):
         if not is_given(opts.http_proxy):
             opts.http_proxy = os.environ.get("HTTPS_PROXY") or os.environ.get("HTTP_PROXY")
 
+        if opts._worker_token:
+            if opts.load_fnc != _DefaultLoadCalc.get_load:
+                logger.warning(
+                    "custom load_fnc is not supported when hosting on Cloud, reverting to default"
+                )
+                opts.load_fnc = _DefaultLoadCalc.get_load
+            if opts.load_threshold != _default_load_threshold:
+                logger.warning(
+                    "custom load_threshold is not supported when hosting on Cloud, reverting to default"
+                )
+                opts.load_threshold = _default_load_threshold
+
         self._opts = opts
         self._loop = loop or asyncio.get_event_loop()
 
@@ -296,7 +309,7 @@ class Worker(utils.EventEmitter[EventTypes]):
         if len(_InferenceRunner.registered_runners) > 0:
             self._inference_executor = ipc.inference_proc_executor.InferenceProcExecutor(
                 runners=_InferenceRunner.registered_runners,
-                initialize_timeout=30,
+                initialize_timeout=opts.initialize_process_timeout,
                 close_timeout=5,
                 memory_warn_mb=2000,
                 memory_limit_mb=0,  # no limit
@@ -345,6 +358,8 @@ class Worker(utils.EventEmitter[EventTypes]):
                     "agent_name": self._opts.agent_name,
                     "worker_type": agent.JobType.Name(self._opts.worker_type.value),
                     "active_jobs": len(self.active_jobs),
+                    "sdk_version": __version__,
+                    "project_type": "python",
                 }
             )
             return web.Response(body=body, content_type="application/json")
@@ -377,7 +392,7 @@ class Worker(utils.EventEmitter[EventTypes]):
         )
 
         if self._opts.multiprocessing_context == "forkserver":
-            plugin_packages = [p.package for p in Plugin.registered_plugins]
+            plugin_packages = [p.package for p in Plugin.registered_plugins] + ["av"]
             logger.info("preloading plugins", extra={"packages": plugin_packages})
             self._mp_ctx.set_forkserver_preload(plugin_packages)
 
